#!/bin/bash
# Download and extract Madrid features from Madrid's city council open data API, sort and renumber
#
# Santiago Crespo 2022

xsltproc -V >/dev/null 2>&1 || { echo "Error: Hace necesitas instalar xsltproc."; echo ""; echo "sudo apt-get install xsltproc -y"; exit 1; }

if [ $# -eq 0 ]; then
    echo "Use: $0 initialID CONFIG_FILE"
    echo "Example:
$0 9000920000 202625"
    exit 1
fi

INITIALID=$1

DIA=`date +%Y-%m-%d`
HORA=`date +%H-%M-%S`

CONFIGFILE="./config/"$2".config"


if [ ! -f "$CONFIGFILE" ]; then
  echo "ERROR: $CONFIGFILE don't exist!"
  exit 1
fi

TAG=`grep ADDTAG $CONFIGFILE | awk -F '=' '{print $2}'`
VALUE=`grep ADDTAG $CONFIGFILE | awk -F '=' '{print $3}'`

mkdir -p data
mkdir -p "source-data/"$TAG$VALUE"/"$DIA"-"$HORA && cd "source-data/"$TAG$VALUE"/"$DIA"-"$HORA || exit 1

CONFIGFILE="../../../config/"$2".config"

echo "Downloading RDF..."
echo "PENDING: if MODIFIEDDATE no ha cambiado desde la última vez, no descargar!"
RDFURL=`grep RDFURL $CONFIGFILE | awk -F '=' '{print $2}'`
wget -nv $RDFURL -O rdf.dcat

SOURCEDATE=`grep modified rdf.dcat | awk -F '>' '{print $2}' | awk -F '<' '{print $1}' | sort -n | tail -n1 | awk -F 'T' '{print $1}'`
XMLURL=`grep '\.xml' rdf.dcat | head -n1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`

echo "Downloading source data..."
wget -nv $XMLURL

CAPA=`ls -1 *xml | awk -F '.' '{print $1}'`

OUT_FILE=`grep "<infoDataset" -A1 $CAPA.xml  | tail -n1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}' | perl -pe 's/ /_/g'`
echo "Procesando conjunto de datos: $OUT_FILE" | perl -pe 's/_/ /g'
# preprocesamiento xml
perl -pe 's/<!\[CDATA\[//g' $CAPA.xml | perl -pe 's/]]>//g' |  perl -pe 's/atributo nombre=//g'  | tr -d '"' |  sed '/^\s*$/d' | sed '1,9d' | perl -pe 's/>\n/XXXXX/g' | perl -pe 's/\n//g' | perl -pe 's/XXXXX/>\n/g' | grep -vi '<tipo>' | grep -v atributos | grep -v DATOSCONTACTOS | grep -v LOCALIZACION | grep -v "atributos idioma" | perl -pe 's/vgnextchannel=bfa48ab43d6bb410VgnVCM100000171f5a0aRCRD&amp;//g' > $OUT_FILE.xml

# ampersands
perl -pe 's/&/--FIXME--/g' $OUT_FILE.xml > foo ; mv foo $OUT_FILE.xml

echo "<catalogo>" > $OUT_FILE-clean.xml
while IFS='' read -r line || [[ -n "$line" ]]; do

  if echo $line | egrep --quiet "^<[A-Z]" ; then
	if echo $line | grep --quiet '<' ; then
	  CLAVE=`echo $line | awk -F '<' '{print $2}' | awk -F '>' '{print $1}'`
      echo $line | perl -pe "s/atributo/"$CLAVE"/g" >> $OUT_FILE-clean.xml
	else
	  echo $line >> $OUT_FILE-clean.xml
	fi
  fi
done < $OUT_FILE.xml

perl -pe 's/<ID-/<\/contenido><contenido><ID-/g' $OUT_FILE-clean.xml | sed '0,/<\/contenido>/s///' > foo ; mv foo $OUT_FILE-clean.xml
echo "</contenido> </catalogo>" >> $OUT_FILE-clean.xml

# Fix para Edificios_de_carácter_monumental-clean.xml (queremos la URL con el PDF)
perl -pe 's/CONTENT-URL/urls-ko/g' Edificios_de_carácter_monumental-clean.xml 2>/dev/null | perl -pe 's/FICHAPDF/CONTENT-URL/g' 2>/dev/null | perl -pe 's/ID-EDIFICIO/ID-ENTIDAD/g' 2>/dev/null > a ; mv a Edificios_de_carácter_monumental-clean.xml 2>/dev/null

echo '<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:template match="/">
  &lt;?xml version="1.0" encoding="UTF-8"?&gt;
  &lt;osm version="0.6" generator="'$0'"&gt;

      <xsl:for-each select="catalogo/contenido">
    &lt;node id="<xsl:value-of select="ID-ENTIDAD"/>" lat="<xsl:value-of select="LATITUD"/>" lon="<xsl:value-of select="LONGITUD"/>" &gt;
        &lt;tag k="name" v="<xsl:value-of select="NOMBRE"/>"/&gt;
        &lt;tag k="description" v="<xsl:value-of select="DESCRIPCION"/> <xsl:value-of select="DESCRIPCION-ENTIDAD"/> <xsl:value-of select="EQUIPAMIENTO"/>"/&gt;
        &lt;tag k="opening_hours" v="<xsl:value-of select="HORARIO"/>"/&gt;
        &lt;tag k="phone" v="+34 <xsl:value-of select="TELEFONO"/>"/&gt;
        &lt;tag k="fax" v="+34 <xsl:value-of select="FAX"/>"/&gt;
        &lt;tag k="addr:street" v="<xsl:value-of select="CLASE-VIAL"/><xsl:text> </xsl:text><xsl:value-of select="NOMBRE-VIA"/>"/&gt;
        &lt;tag k="addr:housenumber" v="<xsl:value-of select="NUM"/>"/&gt;
        &lt;tag k="addr:postcode" v="<xsl:value-of select="CODIGO-POSTAL"/>"/&gt;
        &lt;tag k="addr:city" v="<xsl:value-of select="LOCALIDAD"/>"/&gt;
        &lt;tag k="addr:province" v="<xsl:value-of select="PROVINCIA"/>"/&gt;
        &lt;tag k="addr:suburb" v="<xsl:value-of select="BARRIO"/>"/&gt;
        &lt;tag k="addr:district" v="<xsl:value-of select="DISTRITO"/>"/&gt;
        &lt;tag k="wheelchair" v="<xsl:value-of select="ACCESIBILIDAD"/>"/&gt;
        &lt;tag k="url" v="<xsl:value-of select="CONTENT-URL"/>"/&gt;
        &lt;tag k="email" v="<xsl:value-of select="EMAIL"/>"/&gt;
        &lt;tag k="source" v="Ayuntamiento de Madrid"/&gt;
        &lt;tag k="source:date" v="'$SOURCEDATE'"/&gt;
    &lt;/node&gt;

</xsl:for-each>
</xsl:template>
</xsl:stylesheet>

' > $OUT_FILE.xsl

# Procesado del xsl y xml limpio
rm -f $OUT_FILE.osm
# Fix for Edificios de carácter monumental:
#perl -pe 's/<cata/<contenido> <cata/g' Edificios_de_carácter_monumental-clean.xml > a ; mv a Edificios_de_carácter_monumental-clean.xml
xsltproc $OUT_FILE.xsl $OUT_FILE-clean.xml  > $OUT_FILE.osm

echo "</osm>" >> $OUT_FILE.osm

grep -v '" v="&' $OUT_FILE.osm | grep -v '" v="+34 &' | perl -pe 's/&lt;/</g' | perl -pe 's/&gt;/>/g' | sed '0,/<?xml version="1.0"?>/s///' | perl -pe 's/  <\?/<\?/g' | sed '/^\s*$/d' > tmp0.osm

echo "Adding main tags..."
HORA=`date +%H:%M:%S`
perl -pe 's/id="/version="1" changeset="1" timestamp="'$DIA'T'$HORA'Z" id="/g' "tmp0.osm" | perl -pe 's/ref="-/ref="/g' > tmp.osm

perl -pe 's/>/>\n/g' tmp.osm > tmp2.osm

echo "Removing tags..."
grep "REMOVETAG=" "$CONFIGFILE" | grep -v '"MXASSETNUM"' | perl -pe 's/REMOVETAG=//g' > tags_to_remove
while IFS= read -r tagtoremove
do
  echo "Removing "$tagtoremove"..."
  grep -v '"'$tagtoremove'"' tmp2.osm > a && mv a tmp2.osm
done < tags_to_remove

echo "Adding tags..."
grep "ADDTAG=" "$CONFIGFILE" | perl -pe 's/ADDTAG=//g' > tags_to_add
while IFS= read -r tagstoadd
do
  ADDTAG=`echo $tagstoadd | awk -F '=' '{print $1}'`
  ADDVALUE=`echo $tagstoadd | awk -F '=' '{print $2}'`
  echo "Adding $ADDTAG=$ADDVALUE..."
  perl -pe 's/<tag k="source:date"/<tag k="'$ADDTAG'" v="'$ADDVALUE'"\/> <tag k="source:date"/g' tmp2.osm > a && mv a tmp2.osm
done < tags_to_add

echo "Translating tags..."
grep "MAPTAG=" "$CONFIGFILE" | perl -pe 's/MAPTAG=//g' > tags_to_map
while IFS= read -r tagstomap
do
  OLDTAG=`echo $tagstomap | awk -F '-' '{print $1}'`
  NEWTAG=`echo $tagstomap | awk -F '>' '{print $2}'`
  echo "Mapping $OLDTAG to $NEWTAG..."
  perl -pe 's/"'$OLDTAG'"/"'$NEWTAG'"/g' tmp2.osm > a && mv a tmp2.osm
done < tags_to_map

#echo "Removing some spaces..."
#grep "REMOVESPACES=" "$CONFIGFILE" | perl -pe 's/REMOVESPACES=//g' > tagstoremovespaces
#while IFS= read -r tagtoremovespaces
#do
#  TAGWITHOUTSPACES=`echo $tagtoremovespaces | perl -pe 's/ //g'`
#  echo "Removing spaces from $tagtoremovespaces to $TAGWITHOUTSPACES..."
#  perl -pe 's/"'$tagtoremovespaces'"/"'$TAGWITHOUTSPACES'"/g' tmp2.osm > a && mv a tmp2.osm
#done < tagstoremovespaces


echo "Translating tags removing the original..."
grep "MAPTAGVALUEREMOVINGORIGINAL=" "$CONFIGFILE" | perl -pe 's/MAPTAGVALUEREMOVINGORIGINAL=//g' > tags_to_map_removing_the_original || echo ERROR
while IFS= read -r tagstomap
do
  OLDTAG=`echo $tagstomap | awk -F '=' '{print $1}'`
  OLDVALUE=`echo $tagstomap | awk -F '=' '{print $2}' | perl -pe 's/ /\\\s/g'`
  NEWTAG=`echo $tagstomap | awk -F '>' '{print $2}' | awk -F '=' '{print $1}'`
  NEWVALUE=`echo $tagstomap | awk -F '>' '{print $2}' | awk -F '=' '{print $2}'`
  echo "Mapping $OLDTAG=$OLDVALUE to $NEWTAG=$NEWVALUE removing the original..."
  perl -pe 's/<tag k="'$OLDTAG'" v="'$OLDVALUE'"\/>/<tag k="'$NEWTAG'" v="'$NEWVALUE'"\/>/g' tmp2.osm | perl -pe 's/\\\s/ /g' > a && mv a tmp2.osm
done < tags_to_map_removing_the_original

echo "Translating tags keeping the original..."
echo 'PENDIENTE: cambia "s" por espacios, debería mantener los espacios en el tag o valor'
grep "MAPTAGVALUEKEEPINGORIGINAL=" "$CONFIGFILE" | perl -pe 's/MAPTAGVALUEKEEPINGORIGINAL=//g' > tags_to_map_keeping_the_original || echo ERROR
while IFS= read -r tagstomap
do
  OLDTAG=`echo $tagstomap | awk -F '=' '{print $1}'`
  OLDVALUE=`echo $tagstomap | awk -F '=' '{print $2}' | perl -pe 's/ /\\\s/g'`
  NEWTAG=`echo $tagstomap | awk -F '>' '{print $2}' | awk -F '=' '{print $1}'`
  NEWVALUE=`echo $tagstomap | awk -F '>' '{print $2}' | awk -F '=' '{print $2}'`
  echo "Mapping $OLDTAG=$OLDVALUE to $NEWTAG=$NEWVALUE keeping the original..."
  perl -pe 's/<tag k="'$OLDTAG'" v="'$OLDVALUE'"\/>/<tag k="'$NEWTAG'" v="'$NEWVALUE'"\/> <tag k="'$OLDTAG'" v="'$OLDVALUE'"\/>/g' tmp2.osm | perl -pe 's/\\\s/ /g' > a && mv a tmp2.osm
done < tags_to_map_keeping_the_original

echo "Removing empty tags..."
grep -v '""' tmp2.osm > a && mv a tmp2.osm

echo "Removing elements with no coordinates..."
grep '.' tmp2.osm | perl -pe 's/node>\n/node>/g' | perl -pe 's/node>        <tag/node> \n <node version="1" changeset="1" timestamp="2020-01-23T03:12:00Z" id="999999999" lat="40.4090394" lon="-3.7268155" >       <tag/g' > a && mv a tmp2.osm
echo "Elements with no coordinates and no ID. Added fake data but will be deleted next:"
grep 999999999 tmp2.osm
perl -pe 's/\n//g' tmp2.osm | perl -pe 's/<node/\n<node/g' | perl -pe 's/<way/\n<way/g' | perl -pe 's/sh">        <tag k="name"/sh">\n<tag k="name"/g' | grep -v 'id="999999999"'  | egrep -v '^<tag' | perl -pe 's/>/>\n/g' > a && mv a tmp2.osm

echo "WARNING: CUTTING LINES > 220 char to avoid tag value too long con osmium"
grep -v '.\{220\}' tmp2.osm > a ; mv a tmp2.osm

echo "Sorting and renumbering from $INITIALID..."
echo "Si falla con "OSM tag value is too long", abrir tmp2.osm en JOSM, editar/borrar descripciones o campos largos manualmente, guardar y ejecutar osmium renumber de nuevo: ????"
echo "osmium sort -o tmp-sorted.osm tmp2.osm --overwrite"
osmium sort -o tmp-sorted.osm tmp2.osm --overwrite || exit 1
echo "Si falla por "Node ID twice", abrir tmp-sorted.osm en JOSM, guardar y ejecutar osmium renumber de nuevo:"
echo "osmium renumber --start-id=$INITIALID -o $TAG$VALUE.osm tmp-sorted.osm --overwrite && cp $TAG$VALUE.osm ../../../"
osmium renumber --start-id=$INITIALID -o tmp2.osm tmp-sorted.osm --overwrite || exit 1

if grep -q "SCRIPT=" $CONFIGFILE; then
  echo "Running external script $TAG$VALUE.sh..."
  chmod +x "../../../scripts/$TAG$VALUE.sh"
  cat tmp2.osm | "../../../scripts/$TAG$VALUE.sh" > a && mv a tmp2.osm
fi

echo "Filtrado teléfono y fax vacíos..."
grep -v '" v="+34 "' tmp2.osm > a ; mv a tmp2.osm

mv tmp2.osm $TAG$VALUE.osm

FIRSTID=`grep 'id="' $TAG$VALUE.osm | awk -F 'id="' '{print $2}' | awk -F '"' '{print $1}' | sort -n | head -n1`
LASTID=`grep 'id="' $TAG$VALUE.osm | awk -F 'id="' '{print $2}' | awk -F '"' '{print $1}' | sort -n | tail -n1`
echo "Range used: $FIRSTID-$LASTID"

cp $TAG$VALUE.osm ../../../data/$TAG$VALUE-$CAPA-Ayto-Madrid-API.osm && echo "$TAG$VALUE-$CAPA-Ayto-Madrid-API.osm generated OK!" && exit 0

